/*
 * pilot_connect.c
 *
 * setup a connection to a pilot
 *
 * Copyright (C) 1997 by David Lindes,
 * see the LICENSE file in this directory for licensing information
 *
 * $Id: pilot_connect.c,v 1.10 1997/04/23 03:17:43 lindes Exp $
 *
 */

#include	<pwd.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<unistd.h>
#include	<pi-socket.h>
#include	<sys/errno.h>

#include	"pi-task.h"
#include	"util.h"

void	initialize(PilotConnection	*pilot);
void	parse_config(PilotConnection	*pilot);
void	config_defaults(PilotConnection	*pilot);
void	save_config(PilotConnection	*pilot);
void	init_connection(PilotConnection *pilot);
int	connect_to(PilotConnection *pilot);

extern	int	errno;

void	initialize(PilotConnection	*pilot)
{
    pilot->sockfd	= -1;
    pilot->devicename	= NULL;
    pilot->saveroot	= NULL;
}

void	parse_config(PilotConnection	*pilot)
{
    char		*configfile, buf[BUFSIZ];
    FILE		*config;

    configfile	= pathadd('/', gethomedir(), ".psyncrc", NULL);

    if(!(config	= fopen(configfile,"r")))
    {
	config_defaults(pilot);
	save_config(pilot);
	return;
    }

    while(fgets(buf,BUFSIZ-1,config))
    {
	if(!strncmp(buf,"saveroot:",9))
	{
	    if(pilot->saveroot)
		free(pilot->saveroot);
	    pilot->saveroot	= strdup(&buf[9]);
	    chomp(pilot->saveroot);
	}
	if(!strncmp(buf,"devicename:",11))
	{
	    if(pilot->devicename)
		free(pilot->devicename);
	    pilot->devicename	= strdup(&buf[11]);
	    chomp(pilot->devicename);
	}
	if(!strncmp(buf,"verbosity:",10))
	{
	    pilot->verbosity	= atol(&buf[10]);
	}
	if(!strncmp(buf,"setclock:",9))
	{
	    pilot->setclock	= atol(&buf[9]);
	}
    }
}

void	config_defaults(PilotConnection	*pilot)
{
    printf("Setting config to defaults.\n");

    if(pilot->saveroot)
	free(pilot->saveroot);
    if(pilot->devicename)
	free(pilot->devicename);

    pilot->saveroot	= pathadd('/',gethomedir(),"pilot", NULL);
    pilot->devicename	= strdup("/dev/ttyd2");
    pilot->verbosity	= 0;
    pilot->setclock	= 1;

    if(!(pilot->devicename))
    {
	perror("strdup failed");
	exit(1);
    }
}

void	save_config(PilotConnection	*pilot)
{
    char	*configfile;
    FILE	*config;

    if(!(pilot && pilot->saveroot && pilot->devicename))
    {
	fprintf(stderr,"Asked to save bogus information\n");
	exit(1);
    }

    configfile	= pathadd('/', gethomedir(), ".psyncrc", NULL);

    printf("Saving configuration to %s\n",configfile);

    config	= fopen(configfile,"w");

    if(!config)
    {
	perror("Couldn't open config file");
	exit(1);
    }

    fprintf(config, "saveroot:%s\n",	pilot->saveroot);
    fprintf(config, "devicename:%s\n",	pilot->devicename);
    fprintf(config, "verbosity:%d\n",	pilot->verbosity);
    fprintf(config, "setclock:%d\n",	pilot->setclock);

    if(fclose(config))
    {
	perror("close of configfile failed");
	exit(1);
    }
}

void	init_connection(PilotConnection *pilot)
{
    struct	pi_sockaddr	addr;

    if(!(pilot && pilot->devicename && pilot->saveroot))
    {
	fprintf(stderr,"Attempt to connect without pilot info\n");
	exit(1);
    }

    if(pilot->sockfd >= 0)
	close(pilot->sockfd);

    pilot->sockfd	= pi_socket(PI_PF_SLP, PI_SOCK_STREAM, PI_PF_PADP);
    if(pilot->sockfd < 0)
    {
	perror("pi_socket");
	exit(1);
    }

    addr.sa_family	= PI_AF_SLP;
    addr.port		= 3;
    strncpy(addr.device,pilot->devicename,11);
    addr.device[11]	= '\0';

    if(pi_bind(pilot->sockfd,&addr,sizeof(addr)) == -1)
    {
	perror("pi_bind");
	exit(1);
    }

    if(pi_listen(pilot->sockfd,1) == -1)
    {
	perror("pi_listen");
	exit(1);
    }
}

int	connect_to(PilotConnection *pilot)
{
    printf("Waiting for HotSync button press on %s...\n",
	pilot->devicename);
    pilot->connectionfd	= pi_accept(pilot->sockfd,0,0);
    if(pilot->connectionfd < 0)
    {
	perror("pi_accept");
	if(errno == ETIMEDOUT)
	{
	    fprintf(stderr,"Trying again...\n");
	    return -1;
	}
	perror("pi_accept");
	exit(1);
    }
    return(0);
}
