#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <pi-task.h>
#include <pi-socket.h>
#include <pi-dlp.h>
#include <pi-memo.h>

int	email_callback(PilotConnection *);
int	getcategoryid(char *cat, struct MemoAppInfo *mai);

void	run_registration(void)
{
    register_task('DATA','memo',email_callback);
}

int	email_callback(PilotConnection *pilot)
{
    struct MemoAppInfo mai;
    unsigned char appblock[0xffff], buffer[0xffff];
    int db, i, obcat, smcat;

    smcat	= -1;
    obcat	= -1;

    send_message("E-Mail task running...\n");

    /* Open the Memo's database, store access handle in db */
    if(dlp_OpenDB(pilot->connectionfd, 0, 0x80|0x40, "MemoDB", &db)<0)
    {
	puts("Unable to open MemoDB");
	log_message(pilot,
		"Unabled to open MemoDB for e-mail task\n");
	return(1);
    }

    dlp_ReadAppBlock(pilot->connectionfd, db, 0, appblock, 0xffff);
    unpack_MemoAppInfo(&mai, appblock, 0);

    smcat	= getcategoryid("Sent Mail",&mai);
    obcat	= getcategoryid("Outbox",&mai);

    for (i=0;1;i++) {
	struct Memo	m;
	int		attr, category, j, len;
	time_t		clock;
	FILE		*sendmail;
	ulong_t		id;
				   
	len = dlp_ReadRecordByIndex(pilot->connectionfd, db, i,
		buffer, &id, 0, &attr, &category);
	if(len<0)
	    break;
		
	/* Skip deleted records */
	if((attr & dlpRecAttrDeleted) || (attr & dlpRecAttrArchived))
	    continue;

	/* Skip non E-Mail Outbox records */
	if(strcmp(mai.CategoryName[category],"Outbox"))
	    continue;
		
	unpack_Memo(&m, buffer, len);
	
	time(&clock);

	sendmail	= popen("/usr/lib/sendmail -t","w");
	if(!sendmail)
	{
	    perror("Couldn't pipe sendmail");
	    return(1);
	}

	send_message("Sending an e-mail...\n");
	log_message(pilot, "Sending an e-mail...\n");
	fprintf(sendmail,"Received: from pilot:%s (%s)\n",
	    pilot->uname, pilot->user.username);
	fprintf(sendmail,"\tby email-task $Revision: 1.7 $\n");
	fprintf(sendmail,"\tat %s",ctime(&clock));
	fprintf(sendmail,"From: %s (%s)\n",
	    pilot->uname, pilot->user.username);

	fprintf(sendmail,"%s",m.text);

	pclose(sendmail);

	if(obcat >= 0 && smcat >= 0)
	    dlp_MoveCategory(pilot->connectionfd,db,obcat,smcat);
    }

    /* Close the database */
    dlp_CloseDB(pilot->connectionfd, db);

    return(0);
}

int	getcategoryid(char *cat, struct MemoAppInfo *mai)
{
    int i;

    if(!cat || !(*cat))
	return(-1);

    for(i=0;i<16;i++)
    {
	if(!strcmp(mai->CategoryName[i],cat))
	    return(i);
    }

    return(-1);
}
