/*
 * util.c
 *
 * various utility routines for use in pilot-sync
 *
 * (may be useful in other programs as well, and may be taken
 * as its own unit for such purpose, according to the license)
 *
 * Copyright (C) 1997 by David Lindes,
 * see the LICENSE file in this directory for licensing information
 *
 * $Id: util.c,v 1.4 1997/04/23 03:15:12 lindes Exp $
 *
 */

#include <pwd.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

char	*pathadd(int sepchar, ...)
{
    va_list	args;
    char	*tmp1, *tmp2;
    static char *path = NULL;

    if(path)
    {
	/* path is static so we can clean up after ourselves...
	 * though only if we get called again. */
	free(path);
	path	= NULL;
    }

    va_start(args,sepchar);
    while((tmp1 = va_arg(args,char *)) != NULL)
    {
	tmp2	= realloc(path, (path?strlen(path):0) +
				(tmp1?strlen(tmp1):0) + 1);

	if(!tmp2)
	{
	    perror("realloc failed");
	    exit(1);
	}

	if(path)
	{
	    /* make sure we don't break our string */
	    tmp2[strlen(tmp2) + 1]	= '\0';

	    /* now add a seperator */
	    tmp2[strlen(tmp2)]	= (char)sepchar;
	}
	else
	{
	    *tmp2	= '\0';
	}

	path	= tmp2;

	/* and add the new component */
	strcat(path,tmp1);
    }
    va_end(args);

    /* callers responsibility to free this */
    tmp1	= strdup(path);
    if(!tmp1)
    {
	perror("strdup failed in pathadd");
	exit(1);
    }
    return(tmp1);
}

char	*gethomedir(void)
{
    static char		*homedir = NULL;
    struct passwd	*passwd;

    if(homedir)
	free(homedir);

    homedir	= (char *)getenv("HOME");

    if(!homedir)
    {
	/* get the home directory of the user currently logged in */
	passwd	= getpwnam(getlogin());

	if(!passwd)
	{
	    fprintf(stderr,
		"Couldn't determine home directory.  Exiting.\n");
	    exit(1);
	}

	homedir	= strdup(passwd->pw_dir);
    }
    else
    {
	homedir	= strdup(homedir);
    }

    return(homedir);
}

void	chomp(char *s)
{
    int	length;

    if(!s)
	return;
    
    length	= strlen(s);

    while(s[--length] == '\n' || s[length] == '\r')
	s[length]	= '\0';
}

void	verify_alloc(char *s)
{
    if(!s)
    {
	perror("alloc failed");
	exit(1);
    }
}
